"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useFund = void 0;
const ethers_1 = require("ethers");
const react_1 = require("react");
const contexts_1 = require("../contexts");
const objects_1 = require("../interfaces/objects");
var EActionTypes;
(function (EActionTypes) {
    EActionTypes["FETCHING"] = "FETCHING";
    EActionTypes["FETCHED"] = "FETCHED";
    EActionTypes["TOKEN_SELECTED"] = "TOKEN_SELECTED";
    EActionTypes["SUCCESS"] = "SUCCESS";
    EActionTypes["ERROR"] = "ERROR";
    EActionTypes["AMOUNT_CHANGED"] = "AMOUNT_CHANGED";
})(EActionTypes || (EActionTypes = {}));
function useFund() {
    const { proxy } = react_1.useContext(contexts_1.StoreContext);
    const initialState = {
        loading: true,
        error: null,
        tokenSelectorOptions: [],
        selectedPaymentToken: undefined,
        resultMessage: new objects_1.ResultMessage(objects_1.EResultStatus.IDLE, ""),
        amount: "1",
    };
    const [state, dispatch] = react_1.useReducer((state, action) => {
        switch (action.type) {
            case EActionTypes.FETCHING:
                return Object.assign(Object.assign({}, state), { loading: true });
            case EActionTypes.FETCHED:
                return Object.assign(Object.assign({}, state), { loading: false, tokenSelectorOptions: action.payload });
            case EActionTypes.ERROR:
                return Object.assign(Object.assign({}, state), { loading: false, error: action.payload });
            case EActionTypes.TOKEN_SELECTED:
                return Object.assign(Object.assign({}, state), { loading: false, selectedPaymentToken: action.payload });
            case EActionTypes.AMOUNT_CHANGED:
                return Object.assign(Object.assign({}, state), { loading: false, amount: action.payload });
            case EActionTypes.ERROR:
                return Object.assign(Object.assign({}, state), { loading: false, error: true, resultMessage: new objects_1.ResultMessage(objects_1.EResultStatus.FAILURE, action.payload) });
            case EActionTypes.SUCCESS:
                return Object.assign(Object.assign({}, state), { loading: false, error: false, resultMessage: new objects_1.ResultMessage(objects_1.EResultStatus.SUCCESS, action.payload) });
            default:
                return Object.assign({}, state);
        }
    }, initialState);
    react_1.useEffect(() => {
        let cancelRequest = false;
        const fetchData = () => __awaiter(this, void 0, void 0, function* () {
            dispatch({ type: EActionTypes.FETCHING });
            try {
                if (cancelRequest)
                    return;
                // get data from proxy
                proxy === null || proxy === void 0 ? void 0 : proxy.getBalances().map((balance) => {
                    // prepare balances
                    dispatch({
                        type: EActionTypes.FETCHED,
                        payload: prepareTokenSelector(balance),
                    });
                });
            }
            catch (error) {
                if (cancelRequest)
                    return;
                dispatch({ type: EActionTypes.ERROR, payload: error.message });
            }
        });
        fetchData();
        proxy === null || proxy === void 0 ? void 0 : proxy.onBalancesChanged.subscribe({
            next: (balance) => {
                if (cancelRequest)
                    return;
                dispatch({
                    type: EActionTypes.FETCHED,
                    payload: prepareTokenSelector(balance),
                });
            },
        });
        return function cleanup() {
            cancelRequest = true;
        };
    }, []);
    const prepareTokenSelector = (balance) => {
        return balance.assets.reduce((acc, assetBalance) => {
            acc.push(new objects_1.PaymentTokenOptionViewModel(assetBalance.assetAddress, assetBalance.assetAddress));
            return acc;
        }, new Array());
    };
    const setSelectedPaymentToken = (selectedOption) => {
        dispatch({
            type: EActionTypes.TOKEN_SELECTED,
            payload: selectedOption,
        });
    };
    const setAmount = (enteredAmount) => {
        dispatch({
            type: EActionTypes.AMOUNT_CHANGED,
            payload: enteredAmount,
        });
    };
    const depositFunds = () => {
        var _a, _b;
        if (!((_a = state.selectedPaymentToken) === null || _a === void 0 ? void 0 : _a.address)) {
            dispatch({
                type: EActionTypes.ERROR,
                payload: "Token address not selected",
            });
        }
        proxy
            .depositFunds((_b = state.selectedPaymentToken) === null || _b === void 0 ? void 0 : _b.address, ethers_1.ethers.utils.parseEther("1"))
            .match((balances) => {
            dispatch({
                type: EActionTypes.SUCCESS,
                payload: "your deposit has succeeded",
            });
        }, (err) => {
            dispatch({
                type: EActionTypes.ERROR,
                payload: err.message || "your deposit has failed",
            });
        });
    };
    const mintTokens = () => {
        proxy.mintTestToken(ethers_1.ethers.utils.parseEther(state.amount || "1")).match(() => {
            dispatch({
                type: EActionTypes.SUCCESS,
                payload: "mint tokens has succeeded",
            });
        }, (err) => {
            dispatch({
                type: EActionTypes.ERROR,
                payload: err.message || "mint tokens has failed",
            });
        });
    };
    return Object.assign(Object.assign({}, state), { setSelectedPaymentToken,
        depositFunds,
        mintTokens,
        setAmount });
}
exports.useFund = useFund;
//# sourceMappingURL=useFund.js.map