"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useLinks = void 0;
const objects_1 = require("@hypernetlabs/objects");
const react_1 = require("react");
const contexts_1 = require("../contexts");
var EActionTypes;
(function (EActionTypes) {
    EActionTypes["FETCHING"] = "FETCHING";
    EActionTypes["FETCHED"] = "FETCHED";
    EActionTypes["ERROR"] = "ERROR";
})(EActionTypes || (EActionTypes = {}));
function useLinks() {
    const { proxy } = react_1.useContext(contexts_1.StoreContext);
    const initialState = {
        loading: true,
        error: null,
        links: [],
    };
    const [state, dispatch] = react_1.useReducer((state, action) => {
        switch (action.type) {
            case EActionTypes.FETCHING:
                return Object.assign(Object.assign({}, state), { loading: true });
            case EActionTypes.FETCHED:
                return Object.assign(Object.assign({}, state), { loading: false, links: action.payload });
            case EActionTypes.ERROR:
                return Object.assign(Object.assign({}, state), { loading: false, error: action.payload });
            default:
                return state;
        }
    }, initialState);
    react_1.useEffect(() => {
        let cancelRequest = false;
        let publicIdentifier = "";
        const fetchData = () => __awaiter(this, void 0, void 0, function* () {
            dispatch({ type: EActionTypes.FETCHING });
            try {
                if (cancelRequest)
                    return;
                // get data from proxy
                proxy
                    .getPublicIdentifier()
                    .andThen((publicIdentifierRes) => {
                    publicIdentifier = publicIdentifierRes;
                    return proxy.getLinks();
                })
                    .map((links) => {
                    console.log("links123: ", links);
                    dispatch({ type: EActionTypes.FETCHED, payload: [...links] });
                });
            }
            catch (error) {
                if (cancelRequest)
                    return;
                dispatch({ type: EActionTypes.ERROR, payload: error.message });
            }
        });
        fetchData();
        proxy.onPullPaymentSent.subscribe({
            next: (payment) => {
                const linksArr = [...state.links];
                // Check if there is a link for this counterparty already
                const paymentLinkIndex = linksArr.findIndex((val) => {
                    const counterPartyAccount = val.counterPartyAccount;
                    return (counterPartyAccount === payment.to ||
                        counterPartyAccount === payment.from);
                });
                if (paymentLinkIndex === -1) {
                    // We need to create a new link for the counterparty
                    const counterPartyAccount = payment.to === publicIdentifier ? payment.from : payment.to;
                    linksArr.push(new objects_1.HypernetLink(counterPartyAccount, [payment], [], [payment], [], [payment]));
                }
                else {
                    // It's for us, we'll need to add it to the payments for the link
                    linksArr[paymentLinkIndex].pullPayments.push(payment);
                }
                dispatch({ type: EActionTypes.FETCHED, payload: [...linksArr] });
            },
        });
        proxy.onPushPaymentSent.subscribe({
            next: (payment) => {
                const linksArr = [...state.links];
                // Check if there is a link for this counterparty already
                const paymentLinkIndex = linksArr.findIndex((val) => {
                    const counterPartyAccount = val.counterPartyAccount;
                    return (counterPartyAccount === payment.to ||
                        counterPartyAccount === payment.from);
                });
                if (paymentLinkIndex === -1) {
                    // We need to create a new link for the counterparty
                    const counterPartyAccount = payment.to === publicIdentifier ? payment.from : payment.to;
                    linksArr.push(new objects_1.HypernetLink(counterPartyAccount, [payment], [payment], [], [payment], []));
                }
                else {
                    // It's for us, we'll need to add it to the payments for the link
                    linksArr[paymentLinkIndex].pushPayments.push(payment);
                }
                dispatch({ type: EActionTypes.FETCHED, payload: [...linksArr] });
            },
        });
        return function cleanup() {
            cancelRequest = true;
        };
    }, []);
    return state;
}
exports.useLinks = useLinks;
//# sourceMappingURL=useLinks.js.map