"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.usePayment = void 0;
const objects_1 = require("@hypernetlabs/objects");
const ethers_1 = require("ethers");
const react_1 = require("react");
const contexts_1 = require("../contexts");
const objects_2 = require("../interfaces/objects");
class PaymentTypeOption {
    constructor(typeName, type) {
        this.typeName = typeName;
        this.type = type;
    }
}
var EActionTypes;
(function (EActionTypes) {
    EActionTypes["FETCHING"] = "FETCHING";
    EActionTypes["FETCHED"] = "FETCHED";
    EActionTypes["ERROR"] = "ERROR";
    EActionTypes["TOKEN_SELECTED"] = "TOKEN_SELECTED";
    EActionTypes["COUNTER_PARTY_CHANGED"] = "COUNTER_PARTY_CHANGED";
    EActionTypes["AMOUNT_CHANGED"] = "AMOUNT_CHANGED";
    EActionTypes["EXPIRATION_DATE_CHANGED"] = "EXPIRATION_DATE_CHANGED";
    EActionTypes["REQUIRED_STACK_CHANGED"] = "REQUIRED_STACK_CHANGED";
    EActionTypes["PAYMENT_TYPE_SELECTED"] = "PAYMENT_TYPE_SELECTED";
    EActionTypes["SUCCESS"] = "SUCCESS";
})(EActionTypes || (EActionTypes = {}));
// export function usePayment(): IReducerStateReducer {
function usePayment(initialParams) {
    var _a, _b;
    const { proxy } = react_1.useContext(contexts_1.StoreContext);
    const initialState = {
        loading: true,
        error: false,
        resultMessage: new objects_2.ResultMessage(objects_2.EResultStatus.IDLE, ""),
        tokenSelectorOptions: (initialParams === null || initialParams === void 0 ? void 0 : initialParams.tokenSelectorOptions) || [],
        selectedPaymentToken: ((_b = (_a = initialParams === null || initialParams === void 0 ? void 0 : initialParams.tokenSelectorOptions) === null || _a === void 0 ? void 0 : _a.find((option) => (option === null || option === void 0 ? void 0 : option.address) === (initialParams === null || initialParams === void 0 ? void 0 : initialParams.paymentTokenAddress))) === null || _b === void 0 ? void 0 : _b.address) || undefined,
        counterPartyAccount: (initialParams === null || initialParams === void 0 ? void 0 : initialParams.counterPartyAccount) || "",
        amount: (initialParams === null || initialParams === void 0 ? void 0 : initialParams.amount) || "0",
        expirationDate: (initialParams === null || initialParams === void 0 ? void 0 : initialParams.expirationDate) || "",
        requiredStake: (initialParams === null || initialParams === void 0 ? void 0 : initialParams.requiredStake) || "0",
        merchantUrl: (initialParams === null || initialParams === void 0 ? void 0 : initialParams.merchantUrl) || "http://localhost:5010/",
        paymentType: (initialParams === null || initialParams === void 0 ? void 0 : initialParams.paymentType) || objects_1.EPaymentType.Push,
        paymentTypeOptions: [
            new PaymentTypeOption("Push", objects_1.EPaymentType.Push),
            new PaymentTypeOption("Pull", objects_1.EPaymentType.Pull),
        ],
    };
    const [state, dispatch] = react_1.useReducer((state, action) => {
        switch (action.type) {
            case EActionTypes.FETCHING:
                return Object.assign(Object.assign({}, state), { loading: true });
            case EActionTypes.FETCHED:
                return Object.assign(Object.assign({}, state), { loading: false, tokenSelectorOptions: action.payload });
            case EActionTypes.ERROR:
                return Object.assign(Object.assign({}, state), { loading: false, error: true, resultMessage: new objects_2.ResultMessage(objects_2.EResultStatus.FAILURE, action.payload) });
            case EActionTypes.SUCCESS:
                return Object.assign(Object.assign({}, state), { loading: false, error: false, resultMessage: new objects_2.ResultMessage(objects_2.EResultStatus.SUCCESS, action.payload) });
            case EActionTypes.TOKEN_SELECTED:
                return Object.assign(Object.assign({}, state), { loading: false, selectedPaymentToken: action.payload });
            case EActionTypes.COUNTER_PARTY_CHANGED:
                return Object.assign(Object.assign({}, state), { loading: false, counterPartyAccount: action.payload });
            case EActionTypes.AMOUNT_CHANGED:
                return Object.assign(Object.assign({}, state), { loading: false, amount: action.payload });
            case EActionTypes.EXPIRATION_DATE_CHANGED:
                return Object.assign(Object.assign({}, state), { loading: false, expirationDate: action.payload });
            case EActionTypes.REQUIRED_STACK_CHANGED:
                return Object.assign(Object.assign({}, state), { loading: false, requiredStake: action.payload });
            case EActionTypes.PAYMENT_TYPE_SELECTED:
                return Object.assign(Object.assign({}, state), { loading: false, paymentType: action.payload });
            default:
                return Object.assign({}, state);
        }
    }, initialState);
    react_1.useEffect(() => {
        let cancelRequest = false;
        const fetchData = () => __awaiter(this, void 0, void 0, function* () {
            dispatch({ type: EActionTypes.FETCHING });
            try {
                if (cancelRequest)
                    return;
                // get data from proxy
                proxy === null || proxy === void 0 ? void 0 : proxy.getBalances().map((balance) => {
                    // prepare balances
                    dispatch({
                        type: EActionTypes.FETCHED,
                        payload: prepareTokenSelector(balance),
                    });
                });
            }
            catch (error) {
                if (cancelRequest)
                    return;
                dispatch({ type: EActionTypes.ERROR, payload: error.message });
            }
        });
        fetchData();
        proxy === null || proxy === void 0 ? void 0 : proxy.onBalancesChanged.subscribe({
            next: (balance) => {
                if (cancelRequest)
                    return;
                dispatch({
                    type: EActionTypes.FETCHED,
                    payload: prepareTokenSelector(balance),
                });
            },
        });
        return function cleanup() {
            cancelRequest = true;
        };
    }, []);
    const prepareTokenSelector = (balance) => {
        return balance.assets.reduce((acc, assetBalance) => {
            acc.push(new objects_2.PaymentTokenOptionViewModel(assetBalance.assetAddress, assetBalance.assetAddress));
            return acc;
        }, new Array());
    };
    const setSelectedPaymentToken = (param) => {
        dispatch({
            type: EActionTypes.TOKEN_SELECTED,
            payload: param,
        });
    };
    const setCounterPartyAccount = (param) => {
        dispatch({
            type: EActionTypes.COUNTER_PARTY_CHANGED,
            payload: param,
        });
    };
    const setAmount = (param) => {
        dispatch({
            type: EActionTypes.AMOUNT_CHANGED,
            payload: param,
        });
    };
    const setExpirationDate = (param) => {
        dispatch({
            type: EActionTypes.EXPIRATION_DATE_CHANGED,
            payload: param,
        });
    };
    const setRequiredStake = (param) => {
        dispatch({
            type: EActionTypes.REQUIRED_STACK_CHANGED,
            payload: param,
        });
    };
    const setPaymentType = (param) => {
        dispatch({
            type: EActionTypes.PAYMENT_TYPE_SELECTED,
            payload: param,
        });
    };
    const sendFunds = () => {
        var _a;
        proxy
            .sendFunds(state.counterPartyAccount, ethers_1.utils.parseUnits(state.amount, "wei").toString(), state.expirationDate, ethers_1.utils.parseUnits(state.requiredStake, "wei").toString(), (_a = state.selectedPaymentToken) === null || _a === void 0 ? void 0 : _a.address, state.merchantUrl)
            .match((balances) => {
            dispatch({
                type: EActionTypes.SUCCESS,
                payload: "your fund has succeeded",
            });
        }, (err) => {
            console.log("err: ", err);
            dispatch({
                type: EActionTypes.ERROR,
                payload: err.message || "your fund has failed",
            });
        });
    };
    return Object.assign(Object.assign({}, state), { setSelectedPaymentToken,
        setCounterPartyAccount,
        setAmount,
        setExpirationDate,
        setRequiredStake,
        setPaymentType,
        sendFunds });
}
exports.usePayment = usePayment;
//# sourceMappingURL=usePayment.js.map